﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using System.Web;
using System.IO;
using System.Web.Script.Serialization;

[assembly: WebResource("MvcAjax.MvcAjax.js", "application/x-javascript")]

namespace MvcAjax
{
    internal class PageRequestManager
    {
        private const string PageRequestManagerKey = "MvcAjax_PageRequestManager";

        internal static PageRequestManager GetCurrent(Page page)
        {
            if (page == null)
            {
                throw new ArgumentNullException("page");
            }

            if (!page.Items.Contains(PageRequestManagerKey))
            {
                page.Items[PageRequestManagerKey] = new PageRequestManager(page);
            }

            return (PageRequestManager)page.Items[PageRequestManagerKey];
        }

        internal static bool IsAjaxRequest(HttpRequest request)
        { 
            return request.Headers["MvcAjax_AvailablePanels"] != null;
        }

        private static string[] GetAvailablePanels(HttpRequest request)
        {
            return request.Headers["MvcAjax_AvailablePanels"].Split(';');
        }

        private Page m_page;

        private PageRequestManager(Page page)
        {
            this.m_page = page;
            this.m_page.InitComplete += (sender, args) =>
            {
                this.PageInitCompleted = true;
            };

            if (IsAjaxRequest(this.m_page.Request))
            {
                this.m_page.SetRenderMethodDelegate(new RenderMethod(this.RenderPageCallback));
            }
            else
            {
                this.m_page.PreRender += new EventHandler(this.OnPagePreRender);
            }
        }
        
        private void OnPagePreRender(object sender, EventArgs args)
        {
            this.m_page.Response.Cache.SetCacheability(HttpCacheability.NoCache);

            string scriptUrl = this.m_page.ClientScript.GetWebResourceUrl(typeof(PageRequestManager), "MvcAjax.MvcAjax.js");

            var serializer = new JavaScriptSerializer();
            StringBuilder panelRegisterCodeBuilder = new StringBuilder("window.mvcAjax.panels = {", 1000);
            int processed = 0;
            foreach (var pair in this.MvcAjaxPanels)
            {
                panelRegisterCodeBuilder
                    .Append(serializer.Serialize(pair.Key))
                    .Append(":")
                    .Append("document.getElementById(")
                    .Append(serializer.Serialize(pair.Value.ClientID));

                bool lastOne = (++processed) == this.MvcAjaxPanels.Count;
                panelRegisterCodeBuilder.Append(lastOne ? ")};" : "),");
            }

            string scriptFormat = "<script type='text/javascript' language='javascript' src='{0}'></script>" +
                "<script type='text/javascript' language='javascript'>{1}</script>";
            this.m_page.Controls.Add(new LiteralControl(
                String.Format(scriptFormat, scriptUrl, panelRegisterCodeBuilder.ToString())));
        }

        private void RenderPageCallback(HtmlTextWriter writer, Control control)
        {
            this.m_page.Response.Cache.SetCacheability(HttpCacheability.NoCache);

            HashSet<string> availPanelSet = new HashSet<string>(GetAvailablePanels(this.m_page.Request));
            Dictionary<string, string> result = new Dictionary<string,string>();

            foreach (MvcAjaxPanel panel in this.MvcAjaxPanels.Values)
            {
                if (availPanelSet.Contains(panel.UpdateAreaID))
                {
                    StringWriter stringWriter = new StringWriter();
                    panel.RenderControl(new HtmlTextWriter(stringWriter));

                    result.Add(panel.UpdateAreaID, stringWriter.ToString());
                }
            }

            JavaScriptSerializer serializer = new JavaScriptSerializer();
            writer.Write(serializer.Serialize(result));
        }

        internal bool PageInitCompleted
        {
            get;
            private set;
        }

        private Dictionary<string, MvcAjaxPanel> m_mvcAjaxPanels;
        private Dictionary<string, MvcAjaxPanel> MvcAjaxPanels
        {
            get
            {
                if (this.m_mvcAjaxPanels == null)
                {
                    this.m_mvcAjaxPanels = new Dictionary<string, MvcAjaxPanel>();
                }

                return this.m_mvcAjaxPanels;
            }
        }

        internal void RegisterAjaxPanel(MvcAjaxPanel panel)
        {
            if (this.PageInitCompleted)
            {
                throw new InvalidOperationException("Could not add MvcAjaxPanel after page initializing.");
            }

            if (String.IsNullOrEmpty(panel.UpdateAreaID))
            {
                throw new InvalidOperationException("MvcAjaxPanel's UpdateAreaID could not be null or empty.");
            }

            if (this.MvcAjaxPanels.ContainsKey(panel.UpdateAreaID))
            {
                throw new InvalidOperationException(String.Format(
                    "There's already an UpdateAreaID of value '{0}' in the page.", panel.ID));
            }
            this.MvcAjaxPanels.Add(panel.UpdateAreaID, panel);
        }
    }
}
